import casadi as ca


def derive_Phi_casadi(param):
    # This time we are using casadi without the opti environment Opti()
    # therefore we declare a symbolic casadi variable like this
    u = ca.SX.sym('u',param['N'])

    # build the Phi expression here
    # i.e. an expression that depends on the symbolic variable U
    # you can start from param.x0 and then modify this until you
    # have expressed the complete Phi
    Phi_expr = param['x0']
    Phi_expr = Phi_expr + ...
        
    # TODO: Build expression for phi using the symbolic U


    # After having build Phi_expr we use it to declare a casadi function
    # the function will return both the function value and its jacobian
    # this function can be evaluated as
    # [F, J] = Phi_func(u_test)
    Phi_func = ca.Function('Phi', [u], [Phi_expr, ca.jacobian(Phi_expr, u)])

    # by default a casadi function returns a casadi matrix, which will 
    # cause certain incompatiblity with NumPy arrays. So we have to 
    # change its return type by calling its full() method
    def Phi_casadi(u_test):
        [F, J] = Phi_func(u_test)
        F = F.full()
        J = J.full()
        # F and J are now normal NumPy arrays
        return (F, J)

    return Phi_casadi
